{ **** TRunTimePropManager ****
  v1.0

  This component enables you to select controls at designtime and change any
  property of the selected components at runtime.

  ie : Disable a full group of controls using ONE line of code, without having to include them in panel
       Open/Close your datasets (Tables and Queries) in the correct order using ONE line of code
       Change the font color a full group of controls using ONE line of code
       ...
       use your imagination


  Known limitation : This component doesn't manage notification and therefore
    doesnt detect at design time if you delete a component listed in its list.
    Anyhow, it still checks at LoadTime or before any Editor action at design
    time that the list is correct => no errors

  TESTED on Delphi 4 + BCB 3
  INSTALL : in BCB/Delphi use the menu Component/Install a Component.
            Select this unit and follow the default instructions
            The help file is borland compatible and can be included in the IDE helpsystem

  This component was created by Benoit Voisin, CD2P.
  It is emailware and maybe used in any application as long as you tell me
  you are using it: bvoisin@chez.com.
  You may upgrade this component or use my code has long as you keep my credits
  and send me a copy of your new component(with source please)

  Thanks to the rxteam and TeamB for their help by providing excellent example
  code and answers to my questions.

  Good coding,
  Ben

  Last edited:  03/2000
  Created:      12/1999
}

unit RunTimePropManager;

{$R RunTimePropManager.dcr}

interface


uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,dsgnintf;

type
  TOrder=(orTopToBottom,orBottomToTop);
  TRunTimePropManager = class(TComponent)
  private                        
   FEnablerComponents:TStrings;
   FOrder: TOrder;
   FShowErrorMessages: Boolean;
   procedure SetFEnablerComponents(const Value: TStrings);
  protected
//   procedure Notification(AComponent: TComponent; Operation: TOperation); override;
   procedure Loaded; override;

  public
   procedure UpdateComponentsList;//Updates pointers in the Component List. Must be called if you change the EnablerComponents list.

   procedure SetStrProp(PropertyName, Value: string);//Change the given property of all the EnablerComponents having it with the string Value.
   procedure SetOrdProp(PropertyName: string; Value: LongInt);//Change the given property of all the EnablerComponents having it with the integer Value.
   procedure SetChildStrProp(ChildName,PropertyName, Value: string);//Change the given property of all the ChildName child Component of the EnablerComponents having it with the string Value.
   procedure SetChildOrdProp(ChildName,PropertyName: string; Value: LongInt);//Change the given property of all the ChildName child Component of the EnablerComponents having it with the string Value.
   procedure Enable_Controls(Enable:Boolean);//Short cut for (SetOrdProp('Enabled',Integer(Enable)). Faster too, but works only on TControl descendents
   procedure Activate_DataSets(Active:Boolean);//Short cut for (SetOrdProp('Active',Integer(Active)). Faster too, but works only on TDataSet descendents

   procedure SetOrdPropOfClass(CompClass:TComponentClass;PropertyName: string; Value: LongInt);//Similar to SetOrdProp but acts only works on components descending of the given class. Faster too, since the property location is searched once only.
   procedure SetStrPropOfClass(CompClass:TComponentClass;PropertyName, Value: string);//Similar to SetStrProp but acts only works on components descending of the given class. Faster too, since the property location is searched once only.


   constructor Create(AOwner: TComponent); override;//Creates the manager and initialise a few stuff. You normally do not have to use this method
   destructor Destroy; override;//Cleans up every thing. You normally do not have to use this method
  published
   property EnablerComponents:TStrings read FEnablerComponents write SetFEnablerComponents;//List of Component names. Be sure to call UpdateComponentsList after modifying it
   property Order: TOrder read FOrder write FOrder default orTopToBottom;//Change the order in which the components propertes are set. Usefull to close related DataSet in the inverse order in which they where Activated. TopToBottom : EnablerComponent[0] ... EnablerComponent[n-1]
   property ShowErrorMessages:Boolean read FShowErrorMessages write FShowErrorMessages; //if true, shows a message for each error(component not having the given property, unknown property ...)
  end;
{$IFNDEF RUNONLY}
 procedure Register;
{$ENDIF}

implementation

uses DualList,db,dbtables,stdctrls,typinfo;

{$IFNDEF RUNONLY}
 type
  TRunTimePropManagerEditor = class(TComponentEditor) //Design time editor
   public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

 procedure Register;
 begin
   RegisterComponents('CD2P', [TRunTimePropManager]);
   RegisterComponentEditor(TRunTimePropManager,TRunTimePropManagerEditor);
 end;

 { TRunTimePropManagerEditor }

 procedure TRunTimePropManagerEditor.ExecuteVerb(Index: Integer);
 var
   Enabler: TRunTimePropManager;
   ListeTotale:TStringList;
   i:Integer;
   CompoName:string;
 begin
  Enabler := Component as TRunTimePropManager;
  Enabler.UpdateComponentsList();  //vrifier l'existance de tous les lements
  case Index of
  0:begin
     ListeTotale:=TStringList.Create();
     for i:=Enabler.Owner.ComponentCount-1 downto 0 do
     begin
      CompoName:=Enabler.Owner.Components[i].Name;
      if (Enabler.EnablerComponents.IndexOf(CompoName)=-1) and (not (Enabler.Owner.Components[i] is TRunTimePropManager)) then
       ListeTotale.Add(CompoName);
     end;
     ListeTotale.Sort();

     with TDualListDialog.Create(nil) do
     try
      List1:=ListeTotale;
      List2:=Enabler.EnablerComponents;
      Label1Caption:='Composants non slectionns';
      Label2Caption:='Composants slectionns';

      if (Execute()) then
      begin
       Enabler.EnablerComponents.Assign(List2); //recopier la liste
       Enabler.UpdateComponentsList(); //vrifier l'existance de tous les lements
      end;
     finally
      Free();
      ListeTotale.Free();
     end;
     Designer.Modified(); //prevenir l'IDE que le composant a chang
    end;
   1: Enabler.Enable_Controls(true); //tests
   2: Enabler.Enable_Controls(false);//tests
  end;
 end;

 function TRunTimePropManagerEditor.GetVerb(Index: Integer): string;
 begin
   case Index of
     0: Result := 'Editer la liste des composants';
     1: Result := 'Enable les controles';
     2: Result := 'Disable les controles';
     else Result := '';
   end;
 end;

 function TRunTimePropManagerEditor.GetVerbCount: Integer;
 begin
   Result := 3;
 end;

{$ENDIF}


{ TRunTimePropManager }
constructor TRunTimePropManager.Create(AOwner: TComponent);
begin
 inherited;
 FEnablerComponents:=TStringList.Create();
end;

destructor TRunTimePropManager.Destroy;
begin
 FEnablerComponents.Free();
 inherited Destroy;
end;


//***********************************************************
//
// TRunTimePropManager.SetOrdProp 
//
// 
//
// IN: 
// OUT: 
// RQ:
// Auteur: Benoit VOISIN, CD2P
// Cration: 12/1999
// Modifications: 
// ***********************************************************
procedure TRunTimePropManager.SetOrdProp(PropertyName:string;Value:Longint);
var
 ctrl:TComponent;
 i,ind,nb:Integer;
 propInfo:PPropInfo;
begin
 nb:=FEnablerComponents.Count-1;
 for i:=0 to nb do
 try
  if (FOrder=orTopToBottom) then ind:=i //recherche du composant a traiter
   else ind:=nb-i;
  if (FEnablerComponents.Objects[ind]<>nil) then //si le composant existe :
  begin
   ctrl:=TComponent(FEnablerComponents.Objects[ind]);
   propInfo:=GetPropInfo(ctrl.ClassInfo,PropertyName); //recherche de la proprit
   if (propInfo<>nil) then
    TypInfo.SetOrdProp(ctrl,propInfo,Value);           //si elle existe, la fixer
  end;
 except
  on E:Exception do
   if (ShowErrorMessages) then
    ShowMessage('Problme avec le composant'+FEnablerComponents.Strings[i]+#10#13+E.Message);
 end;
end;

procedure TRunTimePropManager.SetStrProp(PropertyName:string;Value:string);
var
 ctrl:TComponent;
 i,ind,nb:Integer;
 propInfo:PPropInfo;
begin
 nb:=FEnablerComponents.Count-1;
 for i:=0 to nb do
 try
  if (FOrder=orTopToBottom) then ind:=i
   else ind:=nb-i;
  if (FEnablerComponents.Objects[ind]<>nil) then
  begin
   ctrl:=TComponent(FEnablerComponents.Objects[ind]);
   propInfo:=GetPropInfo(ctrl.ClassInfo,PropertyName);
   if (propInfo<>nil) then
    TypInfo.SetStrProp(ctrl,propInfo,Value);
  end;
 except
  on E:Exception do
   if (ShowErrorMessages) then
    ShowMessage('Problme avec le composant'+FEnablerComponents.Strings[i]+#10#13+E.Message);
 end;
end;

procedure TRunTimePropManager.SetChildOrdProp(ChildName,
  PropertyName: string; Value: Integer);
var
 ctrl:TComponent;
 child:TObject;
 iPointer:Longint;
 i,ind,nb:Integer;
 propInfo:PPropInfo;
begin
 nb:=FEnablerComponents.Count-1;
 for i:=0 to nb do
 try
  if (FOrder=orTopToBottom) then ind:=i //choix du composant
   else ind:=nb-i;
  if (FEnablerComponents.Objects[ind]<>nil) then
  begin
   ctrl:=TComponent(FEnablerComponents.Objects[ind]);
   propInfo:=GetPropInfo(ctrl.ClassInfo,ChildName); //recherche de la proprit pointant sur le composant fils:
   iPointer:=TypInfo.GetOrdProp(ctrl,propInfo);     //pointeur sur le composant fils
   if (iPointer<>0) then
   begin
    child:=TObject(iPointer);
    propInfo:=GetPropInfo(child.ClassInfo,PropertyName); //recherche de la proprit sur le composant fils
    if (propInfo<>nil) then
     TypInfo.SetOrdProp(child,propInfo,Value); //on fixe la proprit
   end;
  end;
 except
  on E:Exception do
   if (ShowErrorMessages) then
    ShowMessage('Problme avec le composant'+FEnablerComponents.Strings[i]+#10#13+E.Message);
 end;
end;

procedure TRunTimePropManager.SetChildStrProp(ChildName, PropertyName,
  Value: string);
var
 ctrl:TComponent;
 i,ind,nb:Integer;
 propInfo:PPropInfo;
 iPointer:Longint;
 child:TObject;
begin
 nb:=FEnablerComponents.Count-1;
 for i:=0 to nb do
 try
  if (FOrder=orTopToBottom) then ind:=i
   else ind:=nb-i;
  if (FEnablerComponents.Objects[ind]<>nil) then
  begin
   ctrl:=TComponent(FEnablerComponents.Objects[ind]);
   propInfo:=GetPropInfo(ctrl.ClassInfo,ChildName);
   iPointer:=TypInfo.GetOrdProp(ctrl,propInfo);
   if (iPointer<>0) then
   begin
    child:=TObject(iPointer);
    propInfo:=GetPropInfo(child.ClassInfo,PropertyName);
    if (propInfo<>nil) then
     TypInfo.SetStrProp(child,propInfo,Value);
   end;
  end;
 except
  on E:Exception do
   if (ShowErrorMessages) then
    ShowMessage('Problme avec le composant'+FEnablerComponents.Strings[i]+#10#13+E.Message);
 end;
end;

procedure TRunTimePropManager.Enable_Controls(Enable: Boolean);
var
 ctrl:TControl;
 i,ind,nb:Integer;
begin
 nb:=FEnablerComponents.Count-1;
 for i:=0 to nb do
 try
  if (FOrder=orTopToBottom) then ind:=i
   else ind:=nb-i;
  if (FEnablerComponents.Objects[ind] is TControl) then
  begin
   ctrl:=TControl(FEnablerComponents.Objects[ind]);
   ctrl.Enabled:=Enable;
  end;
 except
  on E:Exception do
   if (ShowErrorMessages) then
    ShowMessage('Problme avec le composant'+FEnablerComponents.Strings[i]+#10#13+E.Message);
 end;
end;

procedure TRunTimePropManager.Activate_DataSets(Active: Boolean);
var
 ctrl:TDataSet;
 i,ind,nb:Integer;
// propInfo:PPropInfo;
begin
 nb:=FEnablerComponents.Count-1;
 for i:=0 to nb do
 try
  if (FOrder=orTopToBottom) then ind:=i
   else ind:=nb-i;
  if (FEnablerComponents.Objects[ind] is TDataSet) then
  begin
   ctrl:=TDataSet(FEnablerComponents.Objects[ind]);
   ctrl.Active:=Active;
  end;
 except
  on E:Exception do
   if (ShowErrorMessages) then
    ShowMessage('Problme avec le composant'+FEnablerComponents.Strings[i]+#10#13+E.Message);
 end;
end;

procedure TRunTimePropManager.Loaded;
begin
 UpdateComponentsList(); //on vrifie la liste
end;



procedure TRunTimePropManager.SetFEnablerComponents(
  const Value: TStrings);
begin
  FEnablerComponents.Assign(Value);
  UpdateComponentsList();
end;

procedure TRunTimePropManager.UpdateComponentsList;
var
 component:TComponent;
 i:Integer;
begin
 for i:=FEnablerComponents.Count-1 downto 0 do
 begin
  component:=Owner.FindComponent(FEnablerComponents.Strings[i]);
  if (component=nil) then
   FEnablerComponents.Delete(i)
  else
   FEnablerComponents.Objects[i]:=component;
 end;
end;

procedure TRunTimePropManager.SetOrdPropOfClass(CompClass: TComponentClass;
  PropertyName: string; Value: Integer);
var
 ctrl:TComponent;
 i,ind,nb:Integer;
 propInfo:PPropInfo;
begin
 nb:=FEnablerComponents.Count-1;
 propInfo:=GetPropInfo(CompClass.ClassInfo,PropertyName);

 if (propInfo<>nil) then
 begin
  for i:=0 to nb do
  try
   if (FOrder=orTopToBottom) then ind:=i
    else ind:=nb-i;
   if (FEnablerComponents.Objects[ind]<>nil) then
   begin
    ctrl:=TComponent(FEnablerComponents.Objects[ind]);
    if (ctrl is CompClass) then
     TypInfo.SetOrdProp(ctrl,propInfo,Value);
   end;
  except
  on E:Exception do
   if (ShowErrorMessages) then
    ShowMessage('Problme avec le composant'+FEnablerComponents.Strings[i]+#10#13+E.Message);
  end;
 end
 else
  if (ShowErrorMessages) then
   ShowMessage('Proprit inconnue:'+PropertyName+' dans la class '+CompClass.ClassName);
end;

procedure TRunTimePropManager.SetStrPropOfClass(CompClass: TComponentClass;
  PropertyName, Value: string);
var
 ctrl:TComponent;
 i,ind,nb:Integer;
 propInfo:PPropInfo;
begin
 nb:=FEnablerComponents.Count-1;
 propInfo:=GetPropInfo(CompClass.ClassInfo,PropertyName);

 if (propInfo<>nil) then
 begin
  for i:=0 to nb do
  try
   if (FOrder=orTopToBottom) then ind:=i
    else ind:=nb-i;
   if (FEnablerComponents.Objects[ind]<>nil) then
   begin
    ctrl:=TComponent(FEnablerComponents.Objects[ind]);
    if (ctrl is CompClass) then
     TypInfo.SetStrProp(ctrl,propInfo,Value);
   end;
  except
  on E:Exception do
   if (ShowErrorMessages) then
    ShowMessage('Problme avec le composant'+FEnablerComponents.Strings[i]+#10#13+E.Message);
  end;
 end
 else
  if (ShowErrorMessages) then
   ShowMessage('Proprit inconnue:'+PropertyName+' dans la class '+CompClass.ClassName);
end;

end.
